from __future__ import print_function

import sys

from .interfaces import ILogger


class LogLevel:
    ERROR = 0
    INFO = 1
    DEBUG = 2


class DefaultLogger(ILogger):
    def __init__(self, level):
        self.level = level

    def _out(self, level, msg):
        if level <= self.level:
            print(msg, file=sys.stderr)

    def error(self, msg):
        self._out(LogLevel.ERROR, msg)

    def info(self, msg):
        self._out(LogLevel.INFO, msg)

    def debug(self, msg):
        self._out(LogLevel.DEBUG, msg)


class SilentLogger(ILogger):
    def error(self, msg):
        pass

    def info(self, msg):
        pass

    def debug(self, msg):
        pass



import importlib
import json
import os
import subprocess

from .compat import WIN


class Sentinel(object):
    def __init__(self, name):
        self.name = name

    def __repr__(self):
        return '<{0}>'.format(self.name)


default = Sentinel('default')


def resolve_spec(spec):
    modname, funcname = spec.rsplit('.', 1)
    module = importlib.import_module(modname)
    func = getattr(module, funcname)
    return func


def is_watchdog_supported():
    """ Return ``True`` if watchdog is available."""
    try:
        import watchdog  # noqa: F401
    except ImportError:
        return False
    return True


def is_watchman_supported():
    """ Return ``True`` if watchman is available."""
    if WIN:
        # for now we aren't bothering with windows sockets
        return False

    try:
        sockpath = get_watchman_sockpath()
        return bool(sockpath)
    except Exception:
        return False


def get_watchman_sockpath(binpath='watchman'):
    """ Find the watchman socket or raise."""
    path = os.getenv('WATCHMAN_SOCK')
    if path:
        return path

    cmd = [binpath, '--output-encoding=json', 'get-sockname']
    result = subprocess.check_output(cmd)
    result = json.loads(result)
    return result['sockname']
